import os
import os.path
from os.path import abspath
from os.path import basename
from os.path import dirname
from os.path import isdir
from os.path import isfile
from os.path import join
import sys
from urllib import unquote_plus

import gconf
import gtk

__author__ = "Matej Urbas"
__date__ = "$21-Jun-2010 20:32:58$"

PLUGIN_NAME="Referencer Notes"

REFERENCER_NOTE_EXTRA_FIELD_NAME = "referencer-notes"
NOTES_EXECUTABLE_PATH = "xdg-open"

XOURNAL_EXTENSION=".xoj"

referencer_plugin_info = {
    "author":       __author__,
    "version":      "0.3.0",
    "longname":     "Provides external notes.",
    "ui":
        """
                    <ui>
                        <menubar name='MenuBar'>
                            <menu action='DocMenu'>
                                <placeholder name='PluginDocMenuActions'>
                                    <menuitem action='_plugin_referencernotes_opennote'/>
                                    <menuitem action='_plugin_referencernotes_addnote'/>
                                </placeholder>
                            </menu>
                        </menubar>
                        <toolbar name='ToolBar'>
                            <placeholder name='PluginToolBarActions'>
                                <toolitem action='_plugin_referencernotes_opennote'/>
                                <toolitem action='_plugin_referencernotes_addnote'/>
                            </placeholder>
                        </toolbar>
                        <popup name='DocPopup'>
                            <placeholder name='PluginDocPopupActions'>
                                <menuitem action='_plugin_referencernotes_opennote'/>
                                <menuitem action='_plugin_referencernotes_addnote'/>
                            </placeholder>
                        </popup>
                    </ui>
                    """

}

referencer_plugin_actions = [
    {
        "name":         "_plugin_referencernotes_opennote",
        "label":        "Open Notes",
        "tooltip":      "Opens the associated notes for the selected documents. The path to the associated notes file should be given in the document's extra field '" + REFERENCER_NOTE_EXTRA_FIELD_NAME + "'. If this extra field is missing, the programme will try to find the note in the configured notes directory (it will look for files named like this: '<Configured Notes Folder>/<Document File Name><Configured Extension>').",
        "icon":         "XournalIcon.png",
        "callback":     "open_notes",
        "sensitivity":  "any_document_has_notes",
        "accelerator":  "<control><shift>x"
    },
    {
        "name":         "_plugin_referencernotes_addnote",
        "label":        "Add Notes",
        "tooltip":      "Associate notes to the selected document.",
        "icon":         "XournalAddNoteIcon.png",
        "callback":     "add_notes",
        "accelerator":  "<control><shift>y"
    }
]

###
### BEGIN: Some utility definitions...
###

def get_absolute_path(file):
    """
    Returns the name of the file relative to the location of this script.
    """
    return abspath(join(dirname(__file__), file))

CONFIG_DIALOG_GLADE_FILE = get_absolute_path("ReferencerNotesConfig.glade")

GCONF_DIR = "/apps/si/urbas/ReferencerNotes"
GCONF_KEY_NOTES_DIRECTORY = GCONF_DIR + "/NotesDirectory"
GCONF_KEY_NOTES_EXTENSION = GCONF_DIR + "/NotesExtenstion"
GCONF_KEY_ADD_NOTES_DIR = GCONF_DIR + "/AddNotesDir"

class Configuration:
    """
    Provides configuration facilities for this plugin.
    """

    _gconf_client = None
    _notes_dir = None
    _notes_extension = None
    _add_notes_dir = None

    def __init__(self):
        try:
            self._gconf_client = gconf.client_get_default()
            self._refresh_notes_dir()
            self._refresh_notes_extension()
            self._refresh_add_notes_dir()
        except:
            show_message("Could not initialise the configuration infrastructure. " + PLUGIN_NAME + " may not work correctly.")

    def _refresh_notes_dir(self):
        """
        Reads the path to the notes directory via GConf.
        """
        self._notes_dir = self._gconf_client.get_string(GCONF_KEY_NOTES_DIRECTORY)
        if self._notes_dir == None or not isdir(self._notes_dir):
            self._notes_dir = os.getcwd()

    def _refresh_notes_extension(self):
        """
        Reads the notes default extension via GConf.
        """
        self._notes_extension = self._gconf_client.get_string(GCONF_KEY_NOTES_EXTENSION)
        if self._notes_extension == None:
            self._notes_extension = XOURNAL_EXTENSION

    def _refresh_add_notes_dir(self):
        """
        Reads the configured path to the directory where to search for notes
        when adding them.
        """
        self._add_notes_dir = self._gconf_client.get_string(GCONF_KEY_ADD_NOTES_DIR)
        if self._add_notes_dir == None or not isdir(self._add_notes_dir):
            self._add_notes_dir = os.getcwd()

    def get_notes_directory(self):
        """
        Returns the configured path of the notes directory.
        """
        return self._notes_dir

    def set_notes_directory(self, dir):
        """
        Sets configuration string for the path to the notes directory.
        """
        if dir == None or not isdir(dir):
            dir = None
        self._gconf_client.set_string(GCONF_KEY_NOTES_DIRECTORY, dir)
        self._notes_dir = dir

    def get_notes_extension(self):
        """
        Returns the configured default extension of notes.
        """
        return self._notes_extension

    def set_notes_extension(self, ext):
        """
        Sets configuration string for the default extension of notes.
        """
        self._gconf_client.set_string(GCONF_KEY_NOTES_EXTENSION, ext)
        self._notes_extension = ext

    def get_add_notes_directory(self):
        """
        This path is used in the 'add new notes' file chooser dialog.
        """
        return self._add_notes_dir

    def set_add_notes_directory(self, dir):
        """
        Remembers the path used in the 'add new notes' file chooser dialog.
        """
        if dir == None or not isdir(dir):
            dir = None
        self._gconf_client.set_string(GCONF_KEY_ADD_NOTES_DIR, dir)
        self._add_notes_dir = dir

# The main configuration interface. All configuration stuff should go through
# it.
config = Configuration()

###
### END: Some utility definitions...
###

def open_notes(library, documents):
    """
    Opens notes associated with the selected documents.
    """
    for document in documents:
        documentPath = get_document_path(document)
        if documentPath != None:
            os.system(NOTES_EXECUTABLE_PATH + " \"" + documentPath + "\"")

def add_notes(library, documents):
    if documents != None and len(documents) > 0:
        document = documents[0]
        fcd = gtk.FileChooserDialog(
            title="Find notes for document '" + document.get_key() + "'",
            action=gtk.FILE_CHOOSER_ACTION_OPEN,
            buttons=(gtk.STOCK_CANCEL,gtk.RESPONSE_CANCEL,gtk.STOCK_OPEN,gtk.RESPONSE_OK)
            )
        try:
            fcd.set_filename(document.get_field(REFERENCER_NOTE_EXTRA_FIELD_NAME))
        except:
            fcd.set_current_folder(config.get_add_notes_directory())
        response = fcd.run()
        if response == gtk.RESPONSE_OK:
            document.set_field(REFERENCER_NOTE_EXTRA_FIELD_NAME, fcd.get_filename())
        fcd.destroy()

def any_document_has_notes(library, documents):
    """
    Checks if any document in the current selection has a note. It returns
    'True' if any only if at least one of the selected documents has an existing
    note.
    """
    for document in documents:
        documentPath = get_document_path(document)
        if documentPath != None:
            return True
    return False

def show_message(message):
    """
    Just shows a dialog (displaying the message).
    """
    dialog = gtk.MessageDialog(None, gtk.DIALOG_MODAL, gtk.MESSAGE_INFO, gtk.BUTTONS_NONE, message)
    dialog.add_button(gtk.STOCK_CLOSE, gtk.RESPONSE_CLOSE)
    dialog.run()
    dialog.destroy()

def construct_document_note_path(document):
    """
    Returns the path to the (potential) notes for the given document. The file
    described by this path might not exist.

    This method simply concatenates paths in the following way:

        <Configured Notes Directory>/<Document Filename><Configured Notes Extension>
    """
    try:
        filePath = unquote_plus(document.get_filename())
        if filePath.startswith("file://"):
            filePath = filePath[7:len(filePath)]
        return join(config.get_notes_directory(), basename(filePath) + config.get_notes_extension())
    except:
        return None


def get_document_path(document):
    """
    Returns the path to the notes for the current document. This method returns
    'None' if and only if the notes do not exist.

    This method first looks up the Xournal note extra field. If this field does
    not exist it tries to find the notes in the notes directory with the default
    notes extension.
    """
    try:
        documentPath = document.get_field(REFERENCER_NOTE_EXTRA_FIELD_NAME)
    except:
        documentPath = construct_document_note_path(document)
    if documentPath != None and isfile(documentPath):
        return documentPath
    return None;

def referencer_config():
    """
    Handles the configuration of this plugin.

    This method is called by Referencer when the user clicks on the 'Configure'
    button in the 'Plugins' dialog. The name of this function is thus fixed and
    should not be changed.
    """
    dlg = ConfigDialog()
    dlg.mainWindow.run()
    dlg.mainWindow.destroy()

class ConfigDialog:
    """
    This class provides the configuration dialog for this plugin.
    """
    def __init__(self):
        # Initialise the UI builder
        self.builder = gtk.Builder()
        self.builder.add_from_file(CONFIG_DIALOG_GLADE_FILE)
        self.builder.connect_signals(self)

        # Fetch all relevant widgets in the dialog
        self.mainWindow = self.builder.get_object("dlgConfig")
        self.fileChooser = self.builder.get_object("fcbNotesPath")
        self.eNotesExtension = self.builder.get_object("eNotesExtension")

        # Initialise the widgets with data...
        self.fileChooser.set_filename(config.get_notes_directory())
        self.eNotesExtension.set_text(config.get_notes_extension())

    def config_ok_click(self, widget, data=None):
        config.set_notes_directory(self.fileChooser.get_filename())
        config.set_notes_extension(self.eNotesExtension.get_text())
        self.mainWindow.destroy()

    def config_cancel_click(self, widget, data=None):
        self.mainWindow.destroy()



if len(sys.argv) > 1 and sys.argv[1] == "configure":
    referencer_config()